#include <Wire.h>
// DS3231 RTC
#include "RTClib.h"

RTC_DS3231 rtc;  // RTC 구조체 변수 선언

// Pin 1-7 is connected to the 7 segments of the display.
int pinA = A1; // a
int pinB = 2; // b
int pinC = 3; // c
int pinD = 4; // d
int pinE = 5; // e
int pinF = 6; // f
int pinG = 7; // g
int dot = A2; // Colon in the middle

// Cooling FAN
int FAN = A4;

int D1 = 8; // rightmost digit_6
int D2 = 9; // 2nd from right digit_5
int D3 = 10; // 3rd from right digit_4
int D4 = 11; // 4th from right digit_3
int D5 = 12; // 5th from right digit_2
int D6 = 13; // 6th from right digit_1
int DD = A3; // Colon control pin

// the setup routine runs once when you press reset:
void setup() {                
  // initialize the digital pins as outputs.
  pinMode(pinA, OUTPUT);     
  pinMode(pinB, OUTPUT);     
  pinMode(pinC, OUTPUT);     
  pinMode(pinD, OUTPUT);     
  pinMode(pinE, OUTPUT);     
  pinMode(pinF, OUTPUT);     
  pinMode(pinG, OUTPUT); 
  pinMode(dot, OUTPUT); 

// Cooling FAN control
  pinMode(FAN, OUTPUT);
  
//  Control line initialization   
  pinMode(D1, OUTPUT);  
  pinMode(D2, OUTPUT);  
  pinMode(D3, OUTPUT);  
  pinMode(D4, OUTPUT);
  pinMode(D5, OUTPUT); 
  pinMode(D6, OUTPUT);
  pinMode(DD, OUTPUT);

  Serial.begin(9600);
  delay(500);
  
  if(!rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }
  
  // Colon is always turn on when control pin is LOW
  digitalWrite(dot, LOW);
}

// Time data varaiables
int hours = 0;
int minutes = 0;
int seconds = 0;

boolean dot_state = false;

void loop() {
    
  int digit_6 = 0;
  int digit_5 = 0;
  int digit_4 = 0;
  int digit_3 = 0;
  int digit_2 = 0;
  int digit_1 = 0;
  
  DateTime now = rtc.now();   // 현재 날짜, 시간을 모듈에서 가져옵니다.

  hours = now.hour();  // 시간 표시
  digit_1 = (hours/10) % 10;
  digit_2 = hours % 10;
  minutes = now.minute();// 분 표시
  digit_3 = (minutes/10) % 10;
  digit_4 = minutes % 10;
  seconds = now.second();// 초 표시
  digit_5 = (seconds/10) % 10;
  digit_6 = seconds % 10;

// Every zero minutes, FAN will be operated for 60 seconds
  if (minutes < 1) {
    digitalWrite(FAN, HIGH);
  }
  else {
    digitalWrite(FAN, LOW);
  }
  
  for(int j=0;j<60;j++) {
    ON_1th();    
    call_disp(digit_1);
    delay(1);
    
    ON_2th();
    call_disp(digit_2);   
    delay(1);
    
    ON_3th(); 
    call_disp(digit_3);    
    delay(1);
      
    ON_4th();
    call_disp(digit_4);  
    delay(1);
  
    ON_5th();
    call_disp(digit_5);  
    delay(1);
  
    ON_6th();
    call_disp(digit_6);  
    delay(1); 
  }

  // Blinking colon
  if (dot_state) { 
    digitalWrite(DD, LOW);
    dot_state = false;
  }
  else { 
    digitalWrite(DD, HIGH);
    dot_state = true;
  }
}

void call_disp(int num) {
  switch (num) {
    case 0:
      disp_0();
      break;
    case 1:
      disp_1();
      break;
    case 2:
      disp_2();
      break;
    case 3:
      disp_3();
      break;
    case 4:
      disp_4();
      break;
    case 5:
      disp_5();
      break;
    case 6:
      disp_6();
      break;
    case 7:
      disp_7();
      break;
    case 8:
      disp_8();
      break;
    case 9:
      disp_9();
      break;
  }
}

void ON_6th() {
  digitalWrite(D1, LOW);
  digitalWrite(D2, HIGH);
  digitalWrite(D3, HIGH);
  digitalWrite(D4, HIGH);
  digitalWrite(D5, HIGH);
  digitalWrite(D6, HIGH);
}

void ON_5th() {
  digitalWrite(D1, HIGH);
  digitalWrite(D2, LOW);
  digitalWrite(D3, HIGH);
  digitalWrite(D4, HIGH);
  digitalWrite(D5, HIGH);
  digitalWrite(D6, HIGH);
}

void ON_4th() {
  digitalWrite(D1, HIGH);
  digitalWrite(D2, HIGH);
  digitalWrite(D3, LOW);
  digitalWrite(D4, HIGH);
  digitalWrite(D5, HIGH);
  digitalWrite(D6, HIGH);
}

void ON_3th() {
  digitalWrite(D1, HIGH);
  digitalWrite(D2, HIGH);
  digitalWrite(D3, HIGH);
  digitalWrite(D4, LOW);
  digitalWrite(D5, HIGH);
  digitalWrite(D6, HIGH);
}

void ON_2th() {
  digitalWrite(D1, HIGH);
  digitalWrite(D2, HIGH);
  digitalWrite(D3, HIGH);
  digitalWrite(D4, HIGH);
  digitalWrite(D5, LOW);
  digitalWrite(D6, HIGH);
}

void ON_1th() {
  digitalWrite(D1, HIGH);
  digitalWrite(D2, HIGH);
  digitalWrite(D3, HIGH);
  digitalWrite(D4, HIGH);
  digitalWrite(D5, HIGH);
  digitalWrite(D6, LOW);
}

void disp_0() {
  digitalWrite(pinA, LOW);   
  digitalWrite(pinB, LOW);   
  digitalWrite(pinC, LOW);   
  digitalWrite(pinD, LOW);   
  digitalWrite(pinE, LOW);   
  digitalWrite(pinF, LOW);   
  digitalWrite(pinG, HIGH); 
}

void disp_1() {
  digitalWrite(pinA, HIGH);   
  digitalWrite(pinB, LOW);   
  digitalWrite(pinC, LOW);   
  digitalWrite(pinD, HIGH);   
  digitalWrite(pinE, HIGH);   
  digitalWrite(pinF, HIGH);   
  digitalWrite(pinG, HIGH); 
}

void disp_2() {
  digitalWrite(pinA, LOW);   
  digitalWrite(pinB, LOW);   
  digitalWrite(pinC, HIGH);   
  digitalWrite(pinD, LOW);   
  digitalWrite(pinE, LOW);   
  digitalWrite(pinF, HIGH);   
  digitalWrite(pinG, LOW); 
}

void disp_3() {
  digitalWrite(pinA, LOW);   
  digitalWrite(pinB, LOW);   
  digitalWrite(pinC, LOW);   
  digitalWrite(pinD, LOW);   
  digitalWrite(pinE, HIGH);   
  digitalWrite(pinF, HIGH);   
  digitalWrite(pinG, LOW); 
}

void disp_4() {
  digitalWrite(pinA, HIGH);   
  digitalWrite(pinB, LOW);   
  digitalWrite(pinC, LOW);   
  digitalWrite(pinD, HIGH);   
  digitalWrite(pinE, HIGH);   
  digitalWrite(pinF, LOW);   
  digitalWrite(pinG, LOW);  
}

void disp_5() {
  digitalWrite(pinA, LOW);   
  digitalWrite(pinB, HIGH);   
  digitalWrite(pinC, LOW);   
  digitalWrite(pinD, LOW);   
  digitalWrite(pinE, HIGH);   
  digitalWrite(pinF, LOW);   
  digitalWrite(pinG, LOW); 
}

void disp_6() {
  digitalWrite(pinA, LOW);   
  digitalWrite(pinB, HIGH);   
  digitalWrite(pinC, LOW);   
  digitalWrite(pinD, LOW);   
  digitalWrite(pinE, LOW);   
  digitalWrite(pinF, LOW);   
  digitalWrite(pinG, LOW); 
}

void disp_7() {
  digitalWrite(pinA, LOW);   
  digitalWrite(pinB, LOW);   
  digitalWrite(pinC, LOW);   
  digitalWrite(pinD, HIGH);   
  digitalWrite(pinE, HIGH);   
  digitalWrite(pinF, HIGH);   
  digitalWrite(pinG, HIGH); 
}

void disp_8() {
  digitalWrite(pinA, LOW);   
  digitalWrite(pinB, LOW);   
  digitalWrite(pinC, LOW);   
  digitalWrite(pinD, LOW);   
  digitalWrite(pinE, LOW);   
  digitalWrite(pinF, LOW);   
  digitalWrite(pinG, LOW); 
}

void disp_9() {
  digitalWrite(pinA, LOW);   
  digitalWrite(pinB, LOW);   
  digitalWrite(pinC, LOW);   
  digitalWrite(pinD, LOW);   
  digitalWrite(pinE, HIGH);   
  digitalWrite(pinF, LOW);   
  digitalWrite(pinG, LOW); 
}
